#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Map/Reduce avec PIG sur cloudera - correction
# 
# Correction.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Données
# 
# On considère le jeu de données suivant : [Localization Data for Person Activity Data Set](https://archive.ics.uci.edu/ml/datasets/Localization+Data+for+Person+Activity) qui ont déjà été récupéré avec le notebook de l'énoncé.

# In[2]:


import os
if "CRTERALAB" in os.environ:
    spl = os.environ["CRTERALAB"].split("**")
    params=dict(server=spl[0], password=spl[1], username=spl[2])
    r = dict
else:
    from pyquickhelper.ipythonhelper import open_html_form
    params={"server":"ws...fr", "username":"x...e", "password":""}
    r = open_html_form(params=params,title="server + credentials", key_save="params")
r


# In[3]:


password = params["password"]
server = params["server"]
username = params["username"]


# In[4]:


get_ipython().run_line_magic('load_ext', 'pyensae')
get_ipython().run_line_magic('load_ext', 'pyenbc')
get_ipython().run_line_magic('remote_open', '')


# <h3 id="exo1">Exercice 1 : GROUP BY</h3>

# In[5]:


import pandas, sqlite3
con = sqlite3.connect("ConfLongDemo_JSI.db3")
df = pandas.read_sql("""SELECT activity, count(*) as nb FROM person GROUP BY activity""", con)
con.close()
df.head()


# Il faut maintenant le faire avec PIG.

# In[6]:


get_ipython().run_cell_magic('PIG', 'solution_groupby.pig', "\nmyinput = LOAD 'ConfLongDemo_JSI.small.example.txt' \n          using PigStorage(',') \n          AS (index:long, sequence, tag, timestamp:long, dateformat, x:double,y:double, z:double, activity) ;\n\ngr = GROUP myinput BY activity ;\navgact = FOREACH gr GENERATE group, COUNT(myinput) ; \n\nSTORE avgact INTO 'ConfLongDemo_JSI.small.group.txt' USING PigStorage() ;\n")


# In[7]:


get_ipython().run_line_magic('pig_submit', 'solution_groupby.pig -r groupby.redirection')


# In[8]:


get_ipython().run_line_magic('remote_cmd', 'tail groupby.redirection.err')


# In[9]:


get_ipython().run_line_magic('remote_cmd', 'hdfs dfs -ls ConfLongDemo_JSI.small.group.txt')


# In[10]:


get_ipython().run_line_magic('remote_cmd', 'hdfs dfs -tail ConfLongDemo_JSI.small.group.txt/part-r-00000')


# <h3 id="exo2">Exercice 2 : JOIN</h3>

# In[11]:


con = sqlite3.connect("ConfLongDemo_JSI.db3")
df = pandas.read_sql("""SELECT person.*, A.nb FROM person INNER JOIN (
                            SELECT activity, count(*) as nb FROM person GROUP BY activity) AS A
                            ON person.activity == A.activity""", con)
con.close()
df.head()


# Idem, maintenant il faut le faire avec PIG.

# In[12]:


get_ipython().run_cell_magic('PIG', 'solution_groupby_join.pig', "\nmyinput = LOAD 'ConfLongDemo_JSI.small.example.txt' \n          using PigStorage(',') \n          AS (index:long, sequence, tag, timestamp:long, dateformat, x:double,y:double, z:double, activity) ;\n\ngr = GROUP myinput BY activity ;\navgact = FOREACH gr GENERATE group, COUNT(myinput) ; \n\njoined = JOIN myinput BY activity, avgact BY group ;\n\nSTORE joined INTO 'ConfLongDemo_JSI.small.group.join.txt' USING PigStorage() ;\n")


# In[13]:


get_ipython().run_line_magic('pig_submit', 'solution_groupby_join.pig -r groupby.join.redirection')


# In[14]:


get_ipython().run_line_magic('remote_cmd', 'tail groupby.join.redirection.err')


# In[15]:


get_ipython().run_line_magic('remote_cmd', 'hdfs dfs -ls ConfLongDemo_JSI.small.group.join.txt')


# In[16]:


get_ipython().run_line_magic('remote_cmd', 'hdfs dfs -tail ConfLongDemo_JSI.small.group.join.txt/part-r-00000')


# <h3 id="prol">Prolongements</h3>
# 
# [PIG](http://pig.apache.org/) n'est pas la seule façon d'exécuter des jobs Map/Reduce. [Hive](https://hive.apache.org/) est un langage dont la syntaxe est très proche de celle du SQL. L'article [Comparing Pig Latin and SQL for Constructing Data Processing Pipelines](https://developer.yahoo.com/blogs/hadoop/comparing-pig-latin-sql-constructing-data-processing-pipelines-444.html) explicite les différences des deux approches.
# 
# **langage haut niveau**
# 
# Ce qu'il faut retenir est que le langage PIG est un langage haut niveau. Le programme est compilé en une séquence d'opérations Map/Reduce transparente pour l'utilisateur. Le temps de développement est très réduit lorsqu'on le compare au même programme écrit en Java. Le compilateur construit un plan d'exécution ([quelques exemples ici](http://chimera.labs.oreilly.com/books/1234000001811/ch07.html#explain)) et infère le nombre de machines requises pour distribuer le job. Cela suffit pour la plupart des besoins, cela nécessite.
# 
# **petits jeux**
# 
# Certains jobs peuvent durer des heures, il est conseillée de les essayer sur des petits jeux de données avant de les faire tourner sur les vrais données. Il est toujours frustrant de s'apercevoir qu'un job a planté au bout de deux heures car une chaîne de caractères est vide et que ce cas n'a pas été prévu.
# 
# Avec ces petits jeux, il est possible de faire tourner et conseillé de tester le job d'abord sur la passerelle ([exécution local](http://archive.cloudera.com/cdh/3/pig/tutorial.html#Running+the+Pig+Scripts+in+Local+Mode)) avant de le lancer sur le cluster. Avec pyensae, il faut ajouter l'option ``-local`` à la commande [pig_submit](http://www.xavierdupre.fr/app/pyensae/helpsphinx/pyensae/remote/magic_remote_ssh.html?highlight=pig_submit#pyensae.remote.magic_remote_ssh.MagicRemoteSSH.pig_submit).
# 
# **concaténer les fichiers divisés**
# 
# Un programme PIG ne produit pas un fichier mais plusieurs fichiers dans un répertoire. La commande [getmerge](http://hadoop.apache.org/docs/r2.3.0/hadoop-project-dist/hadoop-common/FileSystemShell.html) télécharge ces fichiers sur la passerelle et les fusionne en un seul.
# 
# **ordre des lignes**
# 
# Les jobs sont distribués, même en faisant rien (LOAD + STORE), il n'est pas garanti que l'ordre des lignes soit préservé. La probabilié que ce soit le cas est quasi nulle.

# In[17]: